@@ -38,6 +38,30 @@ from ferry.fabric.local import LocalFabric
 from string import Template
 from subprocess import Popen, PIPE
 
+def _get_gateway():
+    cmd = "LC_MESSAGES=C ifconfig ferry0 | grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}'"
+    gw = Popen(cmd, stdout=PIPE, shell=True).stdout.read().strip()
+    
+    cmd = "LC_MESSAGES=C ifconfig ferry0 | grep 'inet addr:' | cut -d: -f4 | awk '{ print $1}'"
+    netmask = Popen(cmd, stdout=PIPE, shell=True).stdout.read().strip()
+    mask = map(int, netmask.split("."))
+    cidr = 1
+    if mask[3] == 0:
+        cidr = 8
+    if mask[2] == 0:
+        cidr *= 2
+    return "%s/%d" % (gw, 32 - cidr)
+
+def _create_bridge():
+    # Check if the Ferry bridge already exists. 
+    # If not go ahead and create a new bridge. 
+    check = Popen("ifconfig ferry0 2> /dev/null", stdout=PIPE, shell=True).stdout.read().strip()
+    if check == "":
+        logging.warning("Creating Ferry bridge 172.18.42.1/16")
+        Popen("brctl addbr ferry0", stdout=PIPE, shell=True).stdout.read().strip()
+        Popen("ip addr add 172.18.42.1/16 dev ferry0", stdout=PIPE, shell=True).stdout.read().strip()
+        Popen("ip link set dev ferry0 up", stdout=PIPE, shell=True).stdout.read().strip()
+
 def _get_download_url():
     if 'DOWNLOAD_URL' in os.environ:
         return os.environ['DOWNLOAD_URL']
@@ -94,13 +118,11 @@ def _has_ferry_user():
 def _supported_arch():
     return struct.calcsize("P") * 8 == 64
 
-def _supported_lxc():
-    output = Popen("(lxc-version 2>/dev/null || lxc-start --version) | sed 's/.* //'", stdout=PIPE, shell=True).stdout.read()
-
+def _supported_tuple(tuple_string, tuple_min):
     # Ignore all non-numeric strings in the
     # versioning information. 
     cleaned = []
-    tuples = output.strip().split(".")[:3]
+    tuples = tuple_string.split(".")[:3]
     for t in tuples: 
         m = re.compile('(\d)*').match(t)
         if m and len(m.groups()) > 0 and m.group(1) != '':
@@ -117,7 +139,20 @@ def _supported_lxc():
     # assumes that lxc-version info is consistent across distributions
     # which may not be true...
     ver = tuple(map(int, cleaned))
-    return ver > (0, 7, 5)
+    return ver > tuple_min, ver
+
+def _get_docker_version():
+    output = Popen("(docker --version 2>/dev/null) | awk '{print $3}'", stdout=PIPE, shell=True).stdout.read()
+    return _supported_tuple(output.strip(), (0, 8, 0))
+
+def _supported_docker():
+    supported, _ = _get_docker_version()
+    return supported
+
+def _supported_lxc():
+    output = Popen("(lxc-version 2>/dev/null || lxc-start --version) | sed 's/.* //'", stdout=PIPE, shell=True).stdout.read()
+    supported, _ = _supported_tuple(output.strip(), (0, 7, 5))
+    return supported
 
 def _supported_python():
     return sys.version_info[0] == 2
@@ -152,7 +187,8 @@ DEFAULT_DOCKER_REPO='ferry'
 GUEST_DOCKER_REPO='ferry-user'
 DEFAULT_FERRY_OWNER='ferry:docker'
 DOCKER_REGISTRY=_get_docker_registry()
-DOCKER_CMD='docker-ferry'
+# DOCKER_CMD='docker-ferry'
+DOCKER_CMD='docker'
 DOCKER_SOCK='unix:////var/run/ferry.sock'
 DOCKER_PID='/var/run/ferry.pid'
 DOCKER_DIR=_get_ferry_dir(server=True)
@@ -309,6 +345,9 @@ class Installer(object):
         if not _supported_lxc():
             return 'You appear to be running an older version of LXC.\nOnly versions > 0.7.5 are supported.'
 
+        if not _supported_docker():
+            return 'You appear to be running an older version of Docker.\nOnly versions > 0.8 are supported.'
+
         if not _has_ferry_user():
             return 'You do not appear to have the \'docker\' group configured. Please create the \'docker\' group and try again.'
 
@@ -809,10 +848,26 @@ class Installer(object):
         return False
         
     def _start_docker_daemon(self, options=None):
+        # Check if the Ferry bridge has been created. 
+        _create_bridge()
+
         # Check if the docker daemon is already running
         try:
             if not self._docker_running():
-                bflag = ''
+                # Use the ferry0 bridge.
+                nflag = ' -b ferry0'
+
+                # Use the LXC backend. This backend option must be specified
+                # for Docker versions greater than 0.9.0. 
+                _, ver = _get_docker_version()
+                if ver > (0, 9, 0):
+                    lflag = ' -e lxc'
+                else:
+                    lflag = ''
+
+                # Figure out which storage backend to use. Right
+                # now we only support BTRFS or DeviceMapper, since
+                # AUFS seems to break on some occasions. 
                 if self._is_running_btrfs():
                     logging.warning("using btrfs backend")
                     bflag = ' -s btrfs'
@@ -832,7 +887,7 @@ class Installer(object):
 
                 # We need to fix this so that ICC is set to false. 
                 icc = ' --icc=true'
-                cmd = 'nohup ' + DOCKER_CMD + ' -d' + ' -H=' + DOCKER_SOCK + ' -g=' + DOCKER_DIR + ' -p=' + DOCKER_PID + dflag + bflag + icc + ' 1>%s  2>&1 &' % DEFAULT_DOCKER_LOG
+                cmd = 'nohup ' + DOCKER_CMD + ' -d' + ' -H=' + DOCKER_SOCK + ' -g=' + DOCKER_DIR + ' -p=' + DOCKER_PID + nflag + dflag + lflag + bflag + icc + ' 1>%s  2>&1 &' % DEFAULT_DOCKER_LOG
                 logging.warning(cmd)
                 Popen(cmd, stdout=PIPE, shell=True)
 
@@ -849,24 +904,9 @@ class Installer(object):
     def _stop_docker_daemon(self, force=False):
         if force or self._docker_running():
             logging.warning("stopping docker daemon")
-            cmd = 'pkill -f docker-ferry'
+            cmd = 'pkill -f ' + DOCKER_CMD
             Popen(cmd, stdout=PIPE, shell=True)
             try:
                 os.remove('/var/run/ferry.sock')
             except OSError:
                 pass
-
-    def _get_gateway(self):
-        cmd = "LC_MESSAGES=C ifconfig drydock0 | grep 'inet addr:' | cut -d: -f2 | awk '{ print $1}'"
-        gw = Popen(cmd, stdout=PIPE, shell=True).stdout.read().strip()
-
-        cmd = "LC_MESSAGES=C ifconfig drydock0 | grep 'inet addr:' | cut -d: -f4 | awk '{ print $1}'"
-        netmask = Popen(cmd, stdout=PIPE, shell=True).stdout.read().strip()
-        mask = map(int, netmask.split("."))
-        cidr = 1
-        if mask[3] == 0:
-            cidr = 8
-        if mask[2] == 0:
-            cidr *= 2
-
-        return "%s/%d" % (gw, 32 - cidr)
